<?php

/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 31/10/19
 * Time: 16.33
 */

namespace FiloBlu\Rma\Helper;

use function count;
use FiloBlu\Rma\Block\Adminhtml\System\Config as Reason;
use function in_array;
use function is_array;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Rma\Api\Data\RmaInterface;
use Magento\Sales\Api\Data\OrderInterface;

use Magento\Sales\Api\OrderItemRepositoryInterface;
use Magento\Store\Model\ScopeInterface;
use Psr\Log\LoggerInterface;

/**
 *
 */
class RmaHelper extends AbstractHelper
{
    /** @var string */
    const XML_PATH_VALIDITY_DAYS = 'sales/rma_validity_settings/validity_days';

    const CACHE_ID = 'rma_reasons';
    const RMA_REASON_CONFIG = 'sales/rma_reason/reason';
    const STANDARD_REASON_ID_FOR_CHANGESIZE = 'wrong_size';
    const PICKUPINSTORE_KEY = 'pickupinstore';
    const ALLOWED_PICKUPINSTORE_STATUSES = 'filoblu_disablerma_section/rma_group/allowed_pickupinstore_order_status';
    const ALLOW_CUSTOMIZED_PRODUCT_REFUND = 'filoblu_disablerma_section/rma_group/allow_customized_product_return';
    const EASYRMA_MATRIXRATE_SOURCE = 'filoblu_disablerma_section/easy_return_matrixrate_group/matrixrate_source';
    const EASYRMA_MATRIXRATE_CONDITION_NAME = 'filoblu_disablerma_section/easy_return_matrixrate_group/condition_name';
    const EASYRMA_CHANGE_SIZE_ORDER_CREATION_STRATEGY = 'filoblu_disablerma_section/rma_group/rma_change_size_order_creation_strategy';
    const EASYRMA_CHANGE_SIZE_SHOW_OSS_SIZES_IN_SELECT = 'filoblu_disablerma_section/rma_group/show_oos_sizes_in_select';
    const XML_PATH_DISALLOWED_RMA_REASON_EMAIL_SENDING = 'filoblu_disablerma_section/rma_group/disallow_email_for_selected_order_status';
    const XML_PATH_SEND_AUTH_EMAIL = 'filoblu_disablerma_section/easy_return_matrixrate_group/send_rma_auth_email';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var Reason
     */
    protected $reason;

    /**
     * @var \FiloBlu\Rma\Helper\SerializerHelper
     */
    protected $serializer;

    /**
     * @var \Magento\Sales\Api\OrderItemRepositoryInterface
     */
    private $orderItemRepository;

    /**
     * RmaHelper constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param ScopeConfigInterface $scopeConfig
     * @param LoggerInterface $logger
     * @param Reason $reason
     * @param \FiloBlu\Rma\Helper\SerializerHelper $serializer
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $orderItemRepository
     */
    public function __construct(
        Context $context,
        ScopeConfigInterface $scopeConfig,
        LoggerInterface $logger,
        Reason $reason,
        SerializerHelper $serializer,
        OrderItemRepositoryInterface $orderItemRepository
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->logger = $logger;
        $this->reason = $reason;
        $this->serializer = $serializer;
        $this->orderItemRepository = $orderItemRepository;
        parent::__construct($context);
    }

    /**
     * @param int $magentoReasonId
     * @return bool
     */
    public function checkIfAutoApproved(int $magentoReasonId)
    {
        $autoApprove = false;

        $value = $this->scopeConfig->getValue(self::RMA_REASON_CONFIG);
        if (empty($value)) {
            return false;
        }

        $mappedReasons = $this->serializer->unserialize($value);
        $standardReasons = $this->reason->getAll();
        foreach ($mappedReasons as $reason) {
            if ((int)$reason['brand_reason'] == $magentoReasonId) {
                foreach ($standardReasons as $stdReason) {
                    if ($stdReason['id'] == $reason['standard_reason'] && $stdReason['automatic_approval']) {
                        $autoApprove = true;
                    }
                }
            }
        }

        return $autoApprove;
    }

    /**
     * @return int
     */
    public function getDefaultChangeSizeMappedReason()
    {
        $reasons = $this->getAllMappedReason();
        if (count($reasons) > 0) {
            foreach ($reasons as $reason) {
                if ($this->checkIfAllowedMagentoChangeSizeReason($reason['brand_reason'], 'brand')) {
                    return (int)$reason['brand_reason'];
                }
            }
        }
        return -1;
    }

    /**
     * @return array
     */
    public function getAllMappedReason()
    {
        $value = $this->scopeConfig->getValue(self::RMA_REASON_CONFIG);

        if (empty($value)) {
            return [];
        }

        return $this->serializer->unserialize($value);
    }

    /**
     * @param int $reasonId
     * @param string|null $reasonType
     * @return bool
     */
    public function checkIfAllowedMagentoChangeSizeReason(int $reasonId, string $reasonType = null)
    {
        if ($this->checkIfItIsMapped($reasonId)) {
            if (in_array(
                self::STANDARD_REASON_ID_FOR_CHANGESIZE,
                $this->getMappedReasonByReasonId($reasonId, 'brand')
            )
            ) {
                return true;
            }

            return false;
        }

        $this->logger->critical("RmaHelper -> Standard Change Size Rma Reason isn't mapped yet!");
        return false;
    }

    /**
     * @param $reasonId
     * @return bool
     */
    public function checkIfItIsMapped($reasonId)
    {
        $value = $this->scopeConfig->getValue(self::RMA_REASON_CONFIG);
        if (empty($value)) {
            return false;
        }

        $mappedReasons = $this->serializer->unserialize($value);
        foreach ($mappedReasons as $reason) {
            if ((int)$reasonId == (int)$reason['brand_reason']) {
                return  true;
            }
        }
        return false;
    }

    /**
     * @param int $reasonId
     * @param string $type
     * @return array
     */
    public function getMappedReasonByReasonId(int $reasonId, string $type = 'standard')
    {
        $value = $this->scopeConfig->getValue(self::RMA_REASON_CONFIG);
        if (empty($value)) {
            return [];
        }

        $mappedReasons = $this->serializer->unserialize($value);
        $exists = false;
        $result = [];
        foreach ($mappedReasons as $reason) {
            if ($type == 'standard') {
                if ($reasonId == $reason['standard_reason']) {
                    $exists = true;
                    $result[] = (int)$reason['brand_reason'];
                }
            } else {
                if (($type == 'brand') && $reasonId == (int)$reason['brand_reason']) {
                    $exists = true;
                    $result[] = $reason['standard_reason'];
                }
            }
        }
        if ($exists) {
            return $result;
        }

        return [];
    }

    /**
     * @param OrderInterface $order
     * @return bool
     */
    public function isValidPickupInStoreOrderStatus(OrderInterface $order)
    {
        if (strpos($order->getShippingMethod() ?? '', self::PICKUPINSTORE_KEY) === false) {
            return true;
        }

        $allowedStatuses = $this->getPickupInStoreAllowedStatusesArray();

        if (in_array($order->getStatus(), $allowedStatuses)) {
            return true;
        }

        return false;
    }

    /**
     * @return array
     */
    protected function getPickupInStoreAllowedStatusesArray()
    {
        $allowedStatus = [];
        $allowedStatusesConfig = $this->scopeConfig->getValue(self::ALLOWED_PICKUPINSTORE_STATUSES);

        if (!is_array($allowedStatusesConfig)) {
            $allowedStatusesConfig = $this->serializer->unserialize($allowedStatusesConfig);
        }

        foreach ($allowedStatusesConfig as $item) {
            if (isset($item['pickupinstore_allowed_status'])) {
                $allowedStatus[] = $item['pickupinstore_allowed_status'];
            }
        }

        return $allowedStatus;
    }

    /**
     * @return mixed
     */
    public function isCustomizedProductRefundAllowed()
    {
        return $this->scopeConfig->getValue(self::ALLOW_CUSTOMIZED_PRODUCT_REFUND);
    }

    /**
     * @return mixed
     */
    public function getEasyReturnMatrixrateSource()
    {
        return $this->scopeConfig->getValue(self::EASYRMA_MATRIXRATE_SOURCE, ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function getEasyReturnMatrixrateConditionName()
    {
        return $this->scopeConfig->getValue(self::EASYRMA_MATRIXRATE_CONDITION_NAME, ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function getChangeSizeOrderCreationStrategy()
    {
        return $this->scopeConfig->getValue(self::EASYRMA_CHANGE_SIZE_ORDER_CREATION_STRATEGY);
    }

    /**
     * @param $store
     * @return bool
     */
    public function getShowOosSizesInSelect($store): bool
    {
        return $this->scopeConfig->isSetFlag(
            self::EASYRMA_CHANGE_SIZE_SHOW_OSS_SIZES_IN_SELECT,
            ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @param $itemId
     * @return int|null
     * @throws InputException
     * @throws NoSuchEntityException
     */
    public function getStoreByOrderItemId($itemId)
    {
        /** @var \Magento\Sales\Model\Order\ItemRepository|\Magento\Sales\Api\OrderItemRepositoryInterface $repository */
        $repository = $this->orderItemRepository;
        return $repository->get($itemId)->getStoreId();
    }

    /**
     * @param $store
     * @return int
     */
    public function getValidityDays($store)
    {
        return (int)$this->scopeConfig->getValue(
            self::XML_PATH_VALIDITY_DAYS,
            ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @return bool
     */
    public function mustSendRmaAuthEmail()
    {
        return $this->scopeConfig->isSetFlag(self::XML_PATH_SEND_AUTH_EMAIL);
    }

    /**
     * @return string|null|array
     */
    public function getDisallowedRmaReasonEmailSending()
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_DISALLOWED_RMA_REASON_EMAIL_SENDING
        );
    }

    /**
     * @param RmaInterface $rma
     * @return bool
     */
    public function isRmaStatusAllowedToSendEmail(RmaInterface $rma): bool
    {
        $disallowedStatuses = $this->getDisallowedRmaStatusEmailSending();

        foreach ($rma->getItems() as $item) {
            if (in_array($item->getReason(), $disallowedStatuses)) {
                return false;
            }
        }

        return true;
    }

    /**
     * @return array
     */
    protected function getDisallowedRmaStatusEmailSending(): array
    {
        $disallowedStatusesArray = $this->getDisallowedRmaReasonEmailSending();

        if (!is_array($disallowedStatusesArray)) {
            $disallowedStatusesArray = $this->serializer->unserialize($disallowedStatusesArray);
        }

        if (!$disallowedStatusesArray) {
            return [];
        }

        $statuses = [];

        foreach ($disallowedStatusesArray as $item) {
            $statuses[] = $item['rma_reson'];
        }

        return $statuses;
    }
}
